@@ -3,16 +3,6 @@
 ### -------------------------------------------------------------------------
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### .subset_SVT_by_logical_array()
-###
-### Returns a vector (atomic or list) of the same type() as 'x'.
-###
-
-.subset_SVT_by_logical_array <- function(x, y)
-    stop("subsetting operation not supported yet")
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### .subset_SVT_by_Lindex()
 ### .subset_SVT_by_Mindex()
@@ -20,7 +10,7 @@
 ### Both return a vector (atomic or list) of the same type() as 'x'.
 ###
 
-.propagate_names_if_1D <- function(ans, x_dimnames, index)
+propagate_names_if_1D <- function(ans, x_dimnames, index)
 {
     if (length(x_dimnames) != 1L)
         return(ans)
@@ -42,10 +32,12 @@
     stopifnot(is.vector(Lindex), is.numeric(Lindex))
     on.exit(free_global_OPBufTree())
     ans <- SparseArray.Call("C_subset_SVT_by_Lindex",
-                            x@dim, x@type, x@SVT, Lindex)
-    .propagate_names_if_1D(ans, dimnames(x), Lindex)
+                            x@dim, x@type, x@SVT, FALSE, Lindex)
+    propagate_names_if_1D(ans, dimnames(x), Lindex)
 }
 
+setMethod("subset_Array_by_Lindex", "SVT_SparseArray", .subset_SVT_by_Lindex)
+
 ### Alright, '.subset_SVT_by_Mindex(x, Mindex)' could just have done:
 ###
 ###     .subset_SVT_by_Lindex(x, Mindex2Lindex(Mindex, dim(x)))
@@ -67,48 +59,65 @@
         if (is.null(x_dimnames))
             stop(wmsg("SparseArray object to subset has no dimnames"))
         ## Subsetting an ordinary array with dimnames on it by a character
-        ## matrix is supported in R base but we don't support this yet for
+        ## matrix is supported in base R but we don't support this yet for
         ## SparseArray objects.
         stop("subsetting a SparseArray object by a character matrix ",
              "is not supported at the moment")
     }
     on.exit(free_global_OPBufTree())
     ans <- SparseArray.Call("C_subset_SVT_by_Mindex",
-                            x@dim, x@type, x@SVT, Mindex)
-    .propagate_names_if_1D(ans, x_dimnames, Mindex)
+                            x@dim, x@type, x@SVT, FALSE, Mindex)
+    propagate_names_if_1D(ans, x_dimnames, Mindex)
+}
+
+setMethod("subset_Array_by_Mindex", "SVT_SparseArray", .subset_SVT_by_Mindex)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### .subset_SVT_as_Rarray()
+###
+### Exomorphic N-dimensional subsetting of an SVT_SparseArray object.
+###
+### Equivalent to 'as.array(.subset_SVT_as_SVT(x, Nindex))' but slightly
+### more efficient.
+
+.subset_SVT_as_Rarray <- function(x, Nindex, ignore.dimnames=FALSE)
+{
+    stopifnot(is(x, "SVT_SparseArray"), isTRUEorFALSE(ignore.dimnames))
+    check_svt_version(x)
+
+    new_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
+    Noffs <- Nindex2Noffs(Nindex)
+    ans <- SparseArray.Call("C_subset_SVT_as_Rarray",
+                            x@dim, x@type, x@SVT, FALSE, Noffs)
+    if (!ignore.dimnames)
+        dimnames(ans) <- S4Arrays:::subset_dimnames_by_Nindex(x@dimnames,
+                                                              Nindex)
+    ans
 }
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### subset_SVT_by_Nindex()
+### .subset_SVT_as_SVT()
 ###
-### In addition to being one of the workhorses behind `[` on an
-### SVT_SparseArray object (see below), this is **the** workhorse behind the
-### extract_sparse_array() and extract_array() methods for SVT_SparseArray
-### objects.
+### Endomorphic N-dimensional subsetting of an SVT_SparseArray object.
 ###
-### 'Nindex' must be an N-index, that is, a list of numeric vectors (or NULLs),
-### one along each dimension in the array to subset. Note that, strictly
-### speaking, the vectors in an N-index are expected to be integer vectors,
-### but subset_SVT_by_Nindex() can handle subscripts of type "double".
-### This differs from the 'index' argument in 'extract_array()' where the
-### subscripts **must** be integer vectors.
+### Like the 'index' argument in 'extract_array()', the 'Nindex' argument
+### must be a **normalized** N-index, that is, a list with one list element
+### per dimension in 'x' where each list element is either a NULL or an
+### integer vector of valid indices along the corresponding dimension in 'x'.
 ###
 ### Returns an SVT_SparseArray object of the same type() as 'x' (endomorphism).
 
-subset_SVT_by_Nindex <- function(x, Nindex, ignore.dimnames=FALSE)
+.subset_SVT_as_SVT <- function(x, Nindex, ignore.dimnames=FALSE)
 {
-    stopifnot(is(x, "SVT_SparseArray"),
-              is.list(Nindex),
-              length(Nindex) == length(x@dim),
-              isTRUEorFALSE(ignore.dimnames))
+    stopifnot(is(x, "SVT_SparseArray"), isTRUEorFALSE(ignore.dimnames))
     check_svt_version(x)
 
-    ## Returns 'new_dim' and 'new_SVT' in a list of length 2.
-    C_ans <- SparseArray.Call("C_subset_SVT_by_Nindex",
-                              x@dim, x@type, x@SVT, Nindex)
-    new_dim <- C_ans[[1L]]
-    new_SVT <- C_ans[[2L]]
+    new_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
+    Noffs <- Nindex2Noffs(Nindex)
+    new_SVT <- SparseArray.Call("C_subset_SVT_as_SVT",
+                                x@dim, x@type, x@SVT, Noffs)
 
     ## Compute 'new_dimnames'.
     if (is.null(dimnames(x)) || ignore.dimnames) {
@@ -124,66 +133,32 @@ subset_SVT_by_Nindex <- function(x, Nindex, ignore.dimnames=FALSE)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Single-bracket subsetting method (`[`) for SVT_SparseArray objects
+### subset_Array_by_Nindex(), extract_array(), and extract_sparse_array()
+### methods for SVT_SparseArray objects
 ###
 
-.subset_SVT_SparseArray <- function(x, i, j, ..., drop=TRUE)
-{
-    if (missing(x))
-        stop(wmsg("'x' is missing"))
-    if (!isTRUEorFALSE(drop))
-        stop(wmsg("'drop' must be TRUE or FALSE"))
-    Nindex <- S4Arrays:::extract_Nindex_from_syscall(sys.call(), parent.frame())
-    nsubscript <- length(Nindex)
-    if (nsubscript == 0L)
-        return(x)  # no-op
-    x_dim <- dim(x)
-    if (nsubscript == 1L && drop) {
-        i <- Nindex[[1L]]
-        if (type(i) == "logical" && identical(x_dim, dim(i)))
-            return(.subset_SVT_by_logical_array(x, i))
-        if (is.matrix(i))
-            return(.subset_SVT_by_Mindex(x, i))
-        if (is.numeric(i))
-            return(.subset_SVT_by_Lindex(x, i))
+setMethod("subset_Array_by_Nindex", "SVT_SparseArray",
+    function(x, Nindex, drop=TRUE)
+    {
+        ans_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
+        if (drop && sum(ans_dim != 1L) <= 1L) {
+            ans <- .subset_SVT_as_Rarray(x, Nindex)
+            return(S4Arrays:::drop_even_if_1D(ans))
+        }
+        ans <- .subset_SVT_as_SVT(x, Nindex)
+        if (drop)
+            ans <- drop(ans)
+        ans
     }
-    if (nsubscript != length(x_dim))
-        stop(wmsg("incorrect number of subscripts"))
-    ## Note that this normalization will coerce the numeric subscripts
-    ## in 'Nindex' to integer. This no longer necessary because now
-    ## subset_SVT_by_Nindex() can handle subscripts of type "double" at
-    ## the C level.
-    ## TODO: Consider using a normalization process here that preserves
-    ## the numeric subscripts.
-    Nindex <- S4Arrays:::normalize_Nindex(Nindex, x)
-    ans <- subset_SVT_by_Nindex(x, Nindex)
-    if (drop)
-        ans <- drop(ans)
-    ans
-}
+)
 
-setMethod("[", "SVT_SparseArray", .subset_SVT_SparseArray)
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### extract_sparse_array() and extract_array() methods for SVT_SparseArray
-### objects
-###
+setMethod("extract_array", "SVT_SparseArray",
+    function(x, index) .subset_SVT_as_Rarray(x, index, ignore.dimnames=TRUE)
+)
 
 ### No need to propagate the dimnames.
 setMethod("extract_sparse_array", "SVT_SparseArray",
-    function(x, index) subset_SVT_by_Nindex(x, index, ignore.dimnames=TRUE)
-)
-
-### Note that the default extract_array() method would do the job but it
-### relies on single-bracket subsetting so would needlessly go thru the
-### complex .subset_SVT_SparseArray() machinery above to finally call
-### subset_SVT_by_Nindex(). It would also propagate the dimnames which
-### extract_array() does not need to do. The method below completely bypasses
-### all this complexity by calling subset_SVT_by_Nindex() directly.
-setMethod("extract_array", "SVT_SparseArray",
-    function(x, index)
-        as.array(subset_SVT_by_Nindex(x, index, ignore.dimnames=TRUE))
+    function(x, index) .subset_SVT_as_SVT(x, index, ignore.dimnames=TRUE)
 )
 
 
